import numpy as np

from daltools import dens
from util import full

from . import tmpdir


class TestDens:

    def setup_method(self):
        self.tmpdir = tmpdir(__file__)
        self.sirifc = self.tmpdir/"SIRIFC"
        np.random.seed(0)

    def test_h1diag(self):
        diref = full.init(
            [
                [0.5370962, 0.33768834, 0.0],
                [0.33768834, 0.21231469, 0.0],
                [0.0, 0.0, 0.0],
            ]
        )

        di, da = dens.h1diag(1, 1, filename=self.tmpdir/"AOONEINT")
        np.testing.assert_almost_equal(di, diref)

    def test_Dab(self):
        na = 2
        nb = 1
        C = full.unit(3)
        da, db = dens.C2Dab(C, na, nb)
        daref = full.init([[1.0, 0.0, 0], [0.0, 1.0, 0.0], [0.0, 0.0, 0.0]])
        dbref = full.init([[1.0, 0.0, 0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]])
        np.testing.assert_almost_equal(da, daref)
        np.testing.assert_almost_equal(db, dbref)

    def test_c1d(self):
        C = np.random.random((3, 2))
        np.testing.assert_allclose(
            dens.C1D(C, 1),
            [
                [0.301196, 0.330805, 0.232507],
                [0.330805, 0.363324, 0.255364],
                [0.232507, 0.255364, 0.179483],
            ],
            atol=1e-6,
        )

    def test_ifc(self):
        di, dv = dens.ifc(filename=self.sirifc)
        np.testing.assert_allclose(
            np.array(di + dv),
            [
                [
                    2.07191150,
                    -0.22368399,
                    -0.00000313,
                    -0.00016714,
                    0.01780186,
                    0.01248206,
                    -0.00822021,
                    -0.00000672,
                    -0.00054138,
                    0.15014399,
                    -0.10963528,
                    -0.10876728,
                ],
                [
                    -0.22368399,
                    0.84346979,
                    0.00001478,
                    -0.00010703,
                    -0.08460485,
                    -0.00226692,
                    -0.17656504,
                    0.00002842,
                    0.00356367,
                    -0.42026218,
                    0.29331527,
                    0.29240082,
                ],
                [
                    -0.00000313,
                    0.00001478,
                    0.74536514,
                    -0.00000004,
                    0.00000084,
                    0.00000093,
                    -0.00000387,
                    0.82191307,
                    0.00000023,
                    0.00000864,
                    -0.00000881,
                    -0.00000848,
                ],
                [
                    -0.00016714,
                    -0.00010703,
                    -0.00000004,
                    0.63099785,
                    0.00013167,
                    -0.00015976,
                    0.00105575,
                    0.00000014,
                    0.15254832,
                    -0.00345828,
                    0.44382789,
                    -0.44949111,
                ],
                [
                    0.01780186,
                    -0.08460485,
                    0.00000084,
                    0.00013167,
                    0.55160669,
                    -0.04388857,
                    -0.00589556,
                    -0.00000636,
                    -0.00039444,
                    -0.58819626,
                    -0.25485196,
                    -0.25042903,
                ],
                [
                    0.01248206,
                    -0.00226692,
                    0.00000093,
                    -0.00015976,
                    -0.04388857,
                    2.11049999,
                    -0.46338323,
                    0.00000118,
                    0.00035710,
                    -0.09573972,
                    0.00949097,
                    0.01038199,
                ],
                [
                    -0.00822021,
                    -0.17656504,
                    -0.00000387,
                    0.00105575,
                    -0.00589556,
                    -0.46338323,
                    2.04785723,
                    -0.00000530,
                    -0.00207408,
                    0.55610424,
                    -0.02116582,
                    -0.02595517,
                ],
                [
                    -0.00000672,
                    0.00002842,
                    0.82191307,
                    0.00000014,
                    -0.00000636,
                    0.00000118,
                    -0.00000530,
                    0.90632235,
                    -0.00000039,
                    0.00001079,
                    -0.00000265,
                    -0.00000298,
                ],
                [
                    -0.00054138,
                    0.00356367,
                    0.00000023,
                    0.15254832,
                    -0.00039444,
                    0.00035710,
                    -0.00207408,
                    -0.00000039,
                    1.90732439,
                    0.00339108,
                    -0.36060959,
                    0.36188631,
                ],
                [
                    0.15014399,
                    -0.42026218,
                    0.00000864,
                    -0.00345828,
                    -0.58819626,
                    -0.09573972,
                    0.55610424,
                    0.00001079,
                    0.00339108,
                    1.03975649,
                    0.11734266,
                    0.11996589,
                ],
                [
                    -0.10963528,
                    0.29331527,
                    -0.00000881,
                    0.44382789,
                    -0.25485196,
                    0.00949097,
                    -0.02116582,
                    -0.00000265,
                    -0.36060959,
                    0.11734266,
                    0.62652073,
                    -0.23950405,
                ],
                [
                    -0.10876728,
                    0.29240082,
                    -0.00000848,
                    -0.44949111,
                    -0.25042903,
                    0.01038199,
                    -0.02595517,
                    -0.00000298,
                    0.36188631,
                    0.11996589,
                    -0.23950405,
                    0.63007216,
                ],
            ],
            atol=1e-7,
        )
